/*
 * gv.h -- Main include file for gv.
 * Copyright (C) 1992  Timothy O. Theisen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Tim Theisen           Systems Programmer
 * Internet: tim@cs.wisc.edu       Department of Computer Sciences
 *     UUCP: uwvax!tim             University of Wisconsin-Madison
 *    Phone: (608)262-0438         1210 West Dayton Street
 *      FAX: (608)262-9777         Madison, WI   53706
 */

#include <stdio.h>
#include <X11/Xos.h>
#include <sys/stat.h>
#include <X11/Intrinsic.h>
#define XtSetFloatArg(arg, n, d) \
    if (sizeof(float) > sizeof(XtArgVal)) { \
	XtSetArg(arg, n, &(d)); \
    } else { \
	XtArgVal *ld = (XtArgVal *)&(d); \
	XtSetArg(arg, n, *ld); \
    }
#include "Gv.h"

/* Application resources */
typedef struct _AppResources {
    Boolean show_title;		/* whether to show title */
    Boolean show_date;		/* whether to show date */
    Boolean auto_center;	/* whether to automatically center the page */
    int wm_horiz_margin;	/* Space taken by window manager */
    int wm_vert_margin;		/* Space taken by window manager */
    int minimum_magstep;	/* smallest magstep allowed */
    int maximum_magstep;	/* largest magstep allowed */
    int magstep;		/* default magstep */
    XtPageOrientation orientation;	/* default orientation */
    String page;		/* first page to show */
    String pagemedia;		/* default page media */
    Boolean force_orientation;	/* use default to override document comments */
    Boolean force_pagemedia;	/* use default to override document comments */
    Boolean swap_landscape;	/* Landscape comment maps to Seascape */
    /* Work arounds for others' bugs */
    Boolean openwindows;	/* whether to work around openwindow bug */
    Boolean ncdwm;		/* whether to work around ncdwm bug */
} AppResources;

extern float	default_xdpi;
extern float	default_ydpi;

extern FILE	*psfile;
extern String	filename;
extern String	oldfilename;
extern int	current_page;
extern int	current_magstep;
extern XtPageOrientation	current_orientation;
extern int	default_pagemedia;
extern int	current_pagemedia;
extern Boolean	force_document_media;
extern int	document_media;
extern int	current_llx;
extern int	current_lly;
extern int	current_urx;
extern int	current_ury;
extern int	base_papersize;
extern Boolean	info_up;
extern int	force_setting;
extern Pixmap	dot_bitmap;
extern Pixmap	menu16_bitmap;
extern Pixmap	tie_fighter_bitmap;
extern String	toc_text;
extern int	toc_length;
extern int	toc_entry_length;
extern int	info_length;
extern time_t	mtime;
extern struct document *doc;
extern struct document *olddoc;
extern Atom	wm_delete_window;
int	catch_Xerror(Display *dpy, XErrorEvent *err);
extern XErrorHandler	old_Xerror;
extern Boolean	dying;
extern XErrorEvent	bomb;

extern XtAppContext app_con;
extern AppResources app_res;

/* Widgets */
extern Widget toplevel;
extern Widget     form;
extern Widget         titlebutton;
extern Widget             titlemenu;
extern Widget         datebutton;
extern Widget             datemenu;
extern Widget         box;
extern Widget             nextbutton;
extern Widget             prevbutton;
extern Widget             showbutton;
extern Widget             magstepbutton;
extern Widget                 magstepmenu;
extern Widget                     *magstepentry;
extern Widget             orientationbutton;
extern Widget                 orientationmenu;
extern Widget                     portraitbutton;
extern Widget                     landscapebutton;
extern Widget                     upsidedownbutton;
extern Widget                     seascapebutton;
extern Widget                     swapbutton;
extern Widget             pagemediabutton;
extern Widget                 pagemediamenu;
extern Widget                     *pagemediaentry;
extern Widget             copyrightbutton;
extern Widget             quitbutton;
extern Widget         toc;
extern Widget         pageview;
extern Widget             page;

/* Popup widgets */
extern Widget infopopup;
extern Widget     infoform;
extern Widget         infotext;
extern Widget         infobutton;
extern Widget copyrightpopup;
extern Widget     copyrightform;
extern Widget         copyrighttext;
extern Widget         copyrightbutton;

/* Callbacks */
void quit_gv(Widget w, XtPointer client_data, XtPointer call_data);
void popup(Widget w, XtPointer client_data, XtPointer call_data);
void prev_page(Widget w, XtPointer client_data, XtPointer call_data);
void this_page(Widget w, XtPointer client_data, XtPointer call_data);
void next_page(Widget w, XtPointer client_data, XtPointer call_data);
void center_page(Widget w, XtPointer client_data, XtPointer call_data);
void mark_page(Widget w, XtPointer client_data, XtPointer call_data);
void unmark_page(Widget w, XtPointer client_data, XtPointer call_data);
void set_magstep(Widget w, XtPointer client_data, XtPointer call_data);
void set_orientation(Widget w, XtPointer client_data, XtPointer call_data);
void swap_landscape(Widget w, XtPointer client_data, XtPointer call_data);
void set_pagemedia(Widget w, XtPointer client_data, XtPointer call_data);
void message(Widget w, XtPointer client_data, XtPointer call_data);
void output(Widget w, XtPointer client_data, XtPointer call_data);
void dismiss(Widget w, XtPointer client_data, XtPointer call_data);
void destroy(Widget w, XtPointer client_data, XtPointer call_data);
void destroy_ghost(Widget w, XtPointer client_data, XtPointer call_data);

/* Actions */
void gv_copyright(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_quit(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_prev(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_show(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_next(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_set_magstep(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_increase_magstep(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_decrease_magstep(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_set_orientation(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_swap_landscape(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_set_pagemedia(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_default(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_force(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_delete_window(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_dismiss(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_scroll_up(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_scroll_down(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_scroll_left(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_scroll_right(Widget w, XEvent *event, String *params, Cardinal *num_params);
void gv_check_file(Widget w, XEvent *event, String *params, Cardinal *num_params);

/* Misc */
void show_page(int number);
Boolean setup_gv(void);
int find_page(String label);
void layout_gv(void);
void magnify(float *dpi, int magstep);
void pscopydoc(FILE *fp);
void positionpopup(Widget w);
Boolean set_new_magstep(void);
Boolean set_new_orientation(int number);
Boolean set_new_pagemedia(int number);
void build_pagemedia_menu(void);
Widget build_label_menu(Widget parent, String name, String label, Pixmap bitmap);
void new_file(int number);
